'use strict';

var AWS = require('aws-sdk');
var prompt = require('prompt');
var colors = require('colors/safe');
var fs = require('fs');
var path = require('path');

var _require = require('lodash'),
    mapKeys = _require.mapKeys,
    camelCase = _require.camelCase;

var config = {};

var COGNITO_REGION = 'eu-west-1';
var COGNITO_FEDERATED_ID_NAME = void 0;

var credentials = void 0;
var s3 = void 0;
var cognitoServiceProvider = void 0;
var cognitoIdentity = void 0;
var iam = void 0;

module.exports = function setup() {
  initialPrompt().then(createBucket).then(putBucketWebsite).then(createBucketPolicy).then(updateCORS).then(createCloudFrontDistribution).then(setUpCognito).then(writeConfigFile).then(createUser).then(confirmUser).catch(function (err) {
    console.log(err);
  });
};

function initialPrompt() {
  return new Promise(function (resolve, reject) {
    prompt.message = '🔧 ';
    console.log('Setup will configure AWS with Cognito, S3 and CloudFront');
    prompt.get([{
      name: 'AWS_ACCESS_KEY_ID',
      description: colors.green('Enter your AWS_ACCESS_KEY_ID'),
      required: true
    }, {
      name: 'AWS_SECRET_ACCESS_KEY',
      description: colors.green('Enter your AWS_SECRET_ACCESS_KEY'),
      required: true
    }, {
      name: 'S3_REGION',
      description: colors.green('Enter your S3_REGION'),
      default: 'eu-west-1',
      required: true
    }, {
      name: 'S3_BUCKET',
      description: colors.green('Enter your S3_BUCKET name'),
      required: true
    }], function (error, result) {
      if (error) {
        console.error('All fields are required');
      } else {
        credentials = {
          accessKeyId: result.AWS_ACCESS_KEY_ID,
          secretAccessKey: result.AWS_SECRET_ACCESS_KEY
        };

        config.S3_REGION = result.S3_REGION;
        config.S3_BUCKET = result.S3_BUCKET;

        s3 = new AWS.S3({
          credentials: credentials,
          region: config.S3_REGION,
          params: {
            Bucket: config.S3_BUCKET,
            ACL: 'public-read'
          }
        });

        cognitoServiceProvider = new AWS.CognitoIdentityServiceProvider({
          credentials: credentials,
          region: COGNITO_REGION
        });

        cognitoIdentity = new AWS.CognitoIdentity({
          credentials: credentials,
          region: COGNITO_REGION
        });

        iam = new AWS.IAM({
          credentials: credentials,
          region: COGNITO_REGION
        });

        COGNITO_FEDERATED_ID_NAME = config.S3_BUCKET.toLowerCase().replace(/\s+/g, '').replace(/[^a-zA-Z]/g, '');

        resolve();
      }
    });
  });
}

function setUpCognito() {
  return createUserPool().then(createUserPoolClient).then(createIdentityPool).then(createUnauthRole).then(createAuthRole).then(setIdentityPoolRoles);
}

function createBucket() {
  return new Promise(function (resolve, reject) {
    s3.headBucket(null, function (err, data) {
      if (err) {
        console.log('Bucket not found…');
        console.log('Creating bucket…');

        s3.createBucket(null, function (err, data) {
          if (err) {
            console.log('Error creating bucket');
            reject(err.message);
          } else {
            console.log('Successfully created bucket');
            resolve();
          }
        });
      } else {
        console.log('Bucket found…');
        resolve();
      }
    });
  });
}

function putBucketWebsite() {
  return new Promise(function (resolve, reject) {
    s3.putBucketWebsite({
      WebsiteConfiguration: {
        IndexDocument: {
          Suffix: 'index.html'
        }
      }
    }, function (err, data) {
      if (err) {
        console.log('Failed to enable bucket website');
        reject(err);
      } else {
        console.log('Successfully enabled bucket website');
        resolve();
      }
    });
  });
}

function createBucketPolicy() {
  return new Promise(function (resolve, reject) {
    s3.putBucketPolicy({
      Policy: '{\n          "Version": "2012-10-17",\n          "Statement": [\n            {\n              "Sid": "PublicReadGetObject",\n              "Effect": "Allow",\n              "Principal": "*",\n              "Action": "s3:GetObject",\n              "Resource": "arn:aws:s3:::' + config.S3_BUCKET + '/*"\n            }\n          ]\n      }'
    }, function (err, data) {
      if (err) {
        console.log('Failed to create bucket policy');
        reject();
      } else {
        console.log('Successfully created bucket policy');
        resolve();
      }
    });
  });
}

function updateCORS() {
  return new Promise(function (resolve, reject) {
    s3.putBucketCors({
      CORSConfiguration: {
        CORSRules: [{
          AllowedHeaders: ['*'],
          AllowedMethods: ['GET', 'PUT', 'POST', 'DELETE'],
          AllowedOrigins: ['*']
        }]
      }
    }, function (err, data) {
      if (err) {
        console.log('Error updating CORS');
        reject(err.message);
      } else {
        console.log('Successfully updated CORS configuration');
        resolve();
      }
    });
  });
}

function createCloudFrontDistribution() {
  return new Promise(function (resolve, reject) {
    var getS3WebsiteEndpoint = function getS3WebsiteEndpoint(region, bucket) {
      return {
        'us-east-1': bucket + '.s3-website-us-east-1.amazonaws.com',
        'us-west-1': bucket + '.s3-website-us-west-1.amazonaws.com',
        'us-west-2': bucket + '.s3-website-us-west-2.amazonaws.com',
        'ap-south-1': bucket + '.s3-website.ap-south-1.amazonaws.com',
        'ap-northeast-2': bucket + '.s3-website.ap-northeast-2.amazonaws.com',
        'ap-southeast-1': bucket + '.s3-website-ap-southeast-1.amazonaws.com',
        'ap-southeast-2': bucket + '.s3-website-ap-southeast-2.amazonaws.com',
        'ap-northeast-1': bucket + '.s3-website-ap-northeast-1.amazonaws.com',
        'eu-central-1': bucket + '.s3-website.eu-central-1.amazonaws.com',
        'eu-west-1': bucket + '.s3-website-eu-west-1.amazonaws.com',
        'sa-east-1': bucket + '.s3-website-sa-east-1.amazonaws.com'
      }[region];
    };

    var cloudfront = new AWS.CloudFront({
      credentials: credentials
    });

    cloudfront.createDistribution({
      DistributionConfig: {
        CallerReference: '' + Date.now(),
        DefaultRootObject: '',
        Origins: {
          Quantity: 1,
          Items: [{
            Id: 'S3-' + config.S3_BUCKET,
            DomainName: getS3WebsiteEndpoint(config.S3_REGION, config.S3_BUCKET),
            OriginPath: '',
            CustomHeaders: {
              Quantity: 0,
              Items: []
            },
            CustomOriginConfig: {
              HTTPPort: 80,
              HTTPSPort: 443,
              OriginProtocolPolicy: 'http-only',
              OriginSslProtocols: {
                Quantity: 3,
                Items: ['TLSv1', 'TLSv1.1', 'TLSv1.2']
              }
            }
          }]
        },
        DefaultCacheBehavior: {
          TargetOriginId: 'S3-' + config.S3_BUCKET,
          ForwardedValues: {
            QueryString: false,
            Cookies: {
              Forward: 'none'
            },
            Headers: {
              Quantity: 0,
              Items: []
            },
            QueryStringCacheKeys: {
              Quantity: 0,
              Items: []
            }
          },
          TrustedSigners: {
            Enabled: false,
            Quantity: 0,
            Items: []
          },
          ViewerProtocolPolicy: 'allow-all',
          MinTTL: 0,
          AllowedMethods: {
            Quantity: 2,
            Items: ['HEAD', 'GET'],
            CachedMethods: {
              Quantity: 2,
              Items: ['HEAD', 'GET']
            }
          },
          SmoothStreaming: false,
          DefaultTTL: 0,
          MaxTTL: 0,
          Compress: true
        },
        CacheBehaviors: {
          Quantity: 1,
          Items: [{
            PathPattern: 'assets/*',
            TargetOriginId: 'S3-' + config.S3_BUCKET,
            ForwardedValues: {
              QueryString: false,
              Cookies: {
                Forward: 'none'
              },
              Headers: {
                Quantity: 0,
                Items: []
              },
              QueryStringCacheKeys: {
                Quantity: 0,
                Items: []
              }
            },
            TrustedSigners: {
              Enabled: false,
              Quantity: 0,
              Items: []
            },
            ViewerProtocolPolicy: 'allow-all',
            MinTTL: 0,
            AllowedMethods: {
              Quantity: 2,
              Items: ['HEAD', 'GET'],
              CachedMethods: {
                Quantity: 2,
                Items: ['HEAD', 'GET']
              }
            },
            SmoothStreaming: false,
            DefaultTTL: 86400,
            MaxTTL: 31536000,
            Compress: true
          }]
        },
        CustomErrorResponses: {
          Quantity: 0,
          Items: []
        },
        Comment: '',
        Logging: {
          Enabled: false,
          IncludeCookies: false,
          Bucket: '',
          Prefix: ''
        },
        PriceClass: 'PriceClass_All',
        Enabled: true,
        WebACLId: '',
        HttpVersion: 'http2'
      }
    }, function (err, data) {
      if (err) {
        console.log(colors.red('Failed to create CloudFront distribution'));
        reject(err);
      } else {
        console.log(colors.green('Successfully created CloudFront distribution with Id:'), data.Id);
        config.CLOUDFRONT_DISTRIBUTION_ID = data.Id;
        console.log('Deploying https://' + data.DomainName + ' - ready in approximately 15 minutes \u2615\uFE0F');
        resolve();
      }
    });
  });
}

function createUserPool() {
  return new Promise(function (resolve, reject) {
    cognitoServiceProvider.createUserPool({
      PoolName: config.S3_BUCKET,
      AliasAttributes: ['phone_number'],
      AutoVerifiedAttributes: ['email'],
      Policies: {
        PasswordPolicy: {
          MinimumLength: 6,
          RequireLowercase: true,
          RequireNumbers: false,
          RequireSymbols: false,
          RequireUppercase: false
        }
      }
      // Custom messages, more security etc can be defined here
    }, function (err, data) {
      if (err) {
        console.log(err);
        reject(err);
      } else {
        console.log('Successfully created user pool');
        config.COGNITO_USER_POOL_ID = data.UserPool.Id;
        resolve(data.UserPool);
      }
    });
  });
}

function createUserPoolClient(userPool) {
  return new Promise(function (resolve, reject) {
    console.log('Creating user pool client (app) for', userPool.Id, '…');

    cognitoServiceProvider.createUserPoolClient({
      ClientName: config.S3_BUCKET,
      UserPoolId: userPool.Id,
      GenerateSecret: false
    }, function (err, data) {
      if (err) {
        console.log('Failed to create user pool client (app)');
        reject(err);
      } else {
        console.log('Successfully created user pool client with id:', data.UserPoolClient.ClientId);
        config.COGNITO_USER_POOL_CLIENT_ID = data.UserPoolClient.ClientId;
        resolve(data.UserPoolClient);
      }
    });
  });
}

function createIdentityPool(userPoolClient) {
  return new Promise(function (resolve, reject) {
    cognitoIdentity.createIdentityPool({
      AllowUnauthenticatedIdentities: true,
      IdentityPoolName: COGNITO_FEDERATED_ID_NAME,
      CognitoIdentityProviders: [{
        ClientId: userPoolClient.ClientId,
        ProviderName: 'cognito-idp.' + COGNITO_REGION + '.amazonaws.com/' + userPoolClient.UserPoolId
      }]
    }, function (err, data) {
      if (err) {
        console.log('Failed to create federated identity pool');
        reject(err);
      } else {
        console.log('Successfully created federated identity pool:', data.IdentityPoolId);
        config.COGNITO_IDENTITY_POOL_ID = data.IdentityPoolId;
        resolve(data.IdentityPoolId);
      }
    });
  });
}

function createUnauthRole(identityPoolId) {
  return new Promise(function (resolve, reject) {
    iam.createRole({
      RoleName: 'flatpackCLI_Cognito_' + COGNITO_FEDERATED_ID_NAME + 'Unauth_Role',
      AssumeRolePolicyDocument: '{\n        "Version": "2012-10-17",\n        "Statement": [\n          {\n            "Effect": "Allow",\n            "Principal": {\n              "Federated": "cognito-identity.amazonaws.com"\n            },\n            "Action": "sts:AssumeRoleWithWebIdentity",\n            "Condition": {\n              "StringEquals": {\n                "cognito-identity.amazonaws.com:aud": "' + identityPoolId + '"\n              },\n              "ForAnyValue:StringLike": {\n                "cognito-identity.amazonaws.com:amr": "unauthenticated"\n              }\n            }\n          }\n        ]\n      }'
    }, function (err, data) {
      if (err) {
        console.log('Failed to create unauth role');
        reject(err);
      } else {
        console.log('Successfully created unauth role:', data.Role.RoleName, data.Role.RoleId);
        console.log('Creating inline policy…');
        iam.putRolePolicy({
          RoleName: data.Role.RoleName,
          PolicyName: 'flatpackCLI_Cognito_' + COGNITO_FEDERATED_ID_NAME + 'Unauth_Role_' + Date.now(),
          PolicyDocument: '{\n            "Version": "2012-10-17",\n            "Statement": [\n              {\n                "Effect": "Allow",\n                "Action": [\n                  "mobileanalytics:PutEvents",\n                  "cognito-sync:*"\n                ],\n                "Resource": [\n                  "*"\n                ]\n              }\n            ]\n          }'
        }, function (err) {
          if (err) {
            console.log('Failed to create inline policy:');
            reject(err);
          } else {
            console.log('Successfully created inline policy');
            resolve({ identityPoolId: identityPoolId, unauthArn: data.Role.Arn });
          }
        });
      }
    });
  });
}

function createAuthRole(_ref) {
  var identityPoolId = _ref.identityPoolId,
      unauthArn = _ref.unauthArn;

  return new Promise(function (resolve, reject) {
    iam.createRole({
      RoleName: 'flatpackCLI_Cognito_' + COGNITO_FEDERATED_ID_NAME + 'Auth_Role',
      AssumeRolePolicyDocument: '{\n        "Version": "2012-10-17",\n        "Statement": [\n          {\n            "Effect": "Allow",\n            "Principal": {\n              "Federated": "cognito-identity.amazonaws.com"\n            },\n            "Action": "sts:AssumeRoleWithWebIdentity",\n            "Condition": {\n              "StringEquals": {\n                "cognito-identity.amazonaws.com:aud": "' + identityPoolId + '"\n              },\n              "ForAnyValue:StringLike": {\n                "cognito-identity.amazonaws.com:amr": "authenticated"\n              }\n            }\n          }\n        ]\n      }'
    }, function (err, data) {
      if (err) {
        console.log('Failed to create auth role');
        reject(err);
      } else {
        console.log('Successfully created auth role:', data.Role.RoleName, data.Role.RoleId);
        console.log('Creating inline policy');
        iam.putRolePolicy({
          RoleName: data.Role.RoleName,
          PolicyName: 'flatpackCLI_Cognito_' + COGNITO_FEDERATED_ID_NAME + 'Auth_Role_' + Date.now(),
          PolicyDocument: '{\n            "Version": "2012-10-17",\n            "Statement": [\n              {\n                "Effect": "Allow",\n                "Action": [\n                  "mobileanalytics:PutEvents",\n                  "cognito-sync:*",\n                  "cognito-identity:*"\n                ],\n                "Resource": [\n                  "*"\n                ]\n              },\n              {\n                "Effect": "Allow",\n                "Action": [\n                  "s3:PutObject",\n                  "s3:GetObject",\n                  "s3:DeleteObject"\n                ],\n                "Resource": ["arn:aws:s3:::' + config.S3_BUCKET + '/*"]\n              },\n              {\n                "Effect": "Allow",\n                "Action": [\n                  "s3:PutBucketWebsite"\n                ],\n                "Resource": ["arn:aws:s3:::' + config.S3_BUCKET + '"]\n              }\n            ]\n          }'
        }, function (err) {
          if (err) {
            console.log('Failed to create inline policy');
            reject(err);
          } else {
            console.log('Successfully created inline policy');
            resolve({ identityPoolId: identityPoolId, unauthArn: unauthArn, authArn: data.Role.Arn });
          }
        });
      }
    });
  });
}

function setIdentityPoolRoles(_ref2) {
  var identityPoolId = _ref2.identityPoolId,
      unauthArn = _ref2.unauthArn,
      authArn = _ref2.authArn;

  return new Promise(function (resolve, reject) {
    cognitoIdentity.setIdentityPoolRoles({
      IdentityPoolId: identityPoolId,
      Roles: {
        unauthenticated: unauthArn,
        authenticated: authArn
      }
    }, function (err, data) {
      if (err) {
        console.log('Failed to set identity pool roles');
        reject(err);
      } else {
        console.log('Successfully set identity pool roles');
        console.log(data);
        resolve();
      }
    });
  });
}

function createUser() {
  return new Promise(function (resolve, reject) {
    prompt.get([{
      name: 'email',
      description: colors.green('Enter your email'),
      required: true
    }, {
      name: 'password',
      hidden: true,
      replace: '•',
      description: colors.green('Enter your password'),
      required: true
    }], function (error, result) {
      if (error) {
        console.error('All fields are required');
      } else {
        cognitoServiceProvider.signUp({
          ClientId: config.COGNITO_USER_POOL_CLIENT_ID,
          Password: result.password,
          Username: result.email,
          UserAttributes: [{
            Name: 'email',
            Value: result.email
          }]
        }, function (err, data) {
          if (err) {
            console.log('Failed to create user');
            reject(err.stack);
          } else {
            console.log('Successfully created user', result.email);
            resolve(result.email);
          }
        });
      }
    });
  });
}

function confirmUser(email) {
  return new Promise(function (resolve, reject) {
    prompt.get([{
      name: 'code',
      description: colors.green('Enter your verification code (sent to ' + email + ')'),
      required: true
    }], function (error, result) {
      if (error) {
        console.error('Please enter your verification code');
      } else {
        cognitoServiceProvider.confirmSignUp({
          ClientId: config.COGNITO_USER_POOL_CLIENT_ID,
          ConfirmationCode: result.code,
          Username: email
        }, function (err, data) {
          if (err) {
            console.log('Failed to confirm email address');
            confirmUser(email).then(resolve);
          } else {
            console.log('Email address confirmed');
            resolve();
          }
        });
      }
    });
  });
}

function writeConfigFile() {
  return new Promise(function (resolve, reject) {
    var camelCased = mapKeys(config, function (val, key) {
      return camelCase(key);
    });
    var contents = JSON.stringify(camelCased, null, 2);

    fs.writeFile(path.join(process.cwd(), 'aws.json'), contents, function (err) {
      if (err) {
        console.log('Could not write aws.json to filesystem');
        reject(err);
      } else {
        console.log('Successfully wrote aws.json to filesystem');
        resolve();
      }
    });
  });
}