'use strict';

var mimeTypes = require('mime-types');
var MemoryFS = require('memory-fs');
var express = require('express');
var webpack = require('webpack');
var webpackDevMiddleware = require('webpack-dev-middleware');
var getWebpackConfig = require('./getWebpackConfig');
var aws = require('./config/aws');

var proxy = require('express-http-proxy');

var icon = '🔧';

var configs = {
  development: getWebpackConfig('development'),
  production: getWebpackConfig('production')
};

var compilers = {
  development: webpack(configs.development),
  production: webpack(configs.production)
};

module.exports = function createDevServer() {
  var fs = new MemoryFS();
  var app = express();

  var middleware = webpackDevMiddleware(compilers.development, {
    stats: {
      chunks: false,
      colors: true
    }
  });

  app.use('/uploads', proxy(aws.s3Bucket + '.s3.amazonaws.com', {
    forwardPath: function forwardPath(req, res) {
      return '/uploads' + require('url').parse(req.url).path;
    }
  }));

  app.post('/build', function (req, res) {
    console.log(icon, 'Compiling production scripts…');

    compilers.production.outputFileSystem = fs;

    compilers.production.run(function (error, stats) {
      if (error) {
        console.log(error);
        return res.status(500).json({ error: error });
      }

      var assets = stats.toJson().assets;

      console.log(icon, 'Compiled', assets.length, 'scripts');

      var files = assets.map(function (asset) {
        console.log(icon, 'Reading ' + asset.name + '\u2026');

        var path = configs.production.output.path + '/' + asset.name;

        var data = fs.readFileSync(path);

        return {
          path: asset.name,
          type: mimeTypes.lookup(asset.name),
          data: data.toString('base64')
        };
      });

      console.log(icon, 'Done!');

      return res.status(200).json({
        files: files,
        stats: stats.toJson()
      });
    });
  });

  app.use(middleware);

  app.use(function (req, res) {
    var notFoundPath = middleware.getFilenameFromUrl('/404.html');

    middleware.fileSystem.readFile(notFoundPath, function (err, data) {
      if (err) {
        res.status(404).send('notFoundRoute not configured');
      } else {
        res.status(404).send(data.toString('utf-8'));
      }
    });
  });

  app.listen(3000, function () {
    console.log('Server listening on http://localhost:3000');
  });
};